import { MockMethod } from 'vite-plugin-mock';
import { resultPageSuccess, resultSuccess } from '../_util';
import { query, ChangeServer, databaseName, serverName } from './query';
import { MsgType, MsgResult, SendManage } from './socket';
// 接收一个sql语句 以及所需的values
// 这里接收第二参数values的原因是可以使用mysql的占位符 '?'
// 比如 query(`select * from my_database where id = ?`, [1])

async function RefreshCharacterList(
  isOnline = 2,
  accountID = '',
  name = '',
  loginIP = '',
  loginMac = '',
) {
  const dbName = databaseName();
  console.log('DBName:' + dbName);
  let queryStr =
    'SELECT c.*,a.banned,a.Password,a.HookSuspicionLevel FROM ' +
    dbName +
    '.characterinfo c left join standard_account.account a on c.accountIndex=a.indexid where 1=1';
  if (isOnline != 2) {
    queryStr += " and c.isOnline='" + isOnline + "'";
  }
  if (accountID != '') {
    queryStr += " and c.accountID='" + accountID + "'";
  }
  if (name != '') {
    queryStr += " and c.name like '%" + name + "%'";
  }
  if (loginIP != '') {
    queryStr += " and c.loginIP = '" + loginIP + "'";
  }
  if (loginMac != '') {
    queryStr += " and c.loginMac = '" + loginMac + "'";
  }
  queryStr += ' limit 10000';
  console.log('QueryStr: ', queryStr);
  const results = await query(queryStr, '');
  //console.log('QueryCount: ', results.length);
  let i: number;
  for (i = 0; i < results.length; i++) {
    const data = results[i];
    if (data.Gender == 1) {
      data.Gender = '女';
    } else {
      data.Gender = '男';
    }
    if (data.Class == 0) {
      data.Class = '战士';
    } else if (data.Class == 1) {
      data.Class = '法师';
    } else {
      data.Class = '道士';
    }
    if (data.IsOnline == 1) {
      data.IsOnline = '是';
    } else {
      data.IsOnline = '否';
    }
    if (data.Banned == 1) {
      data.Banned = '是';
    } else {
      data.Banned = '否';
    }
    //console.log('Data: ', data);
    if (CharacterList.length > i) {
      CharacterList[i] = data;
    } else {
      CharacterList.push(data);
    }
  }
  if (CharacterList.length > results.length) {
    CharacterList.splice(results.length, CharacterList.length - results.length);
  }
  //console.log('ResultCount: ', CharacterList.length);
  return results;
}

const CharacterList: any[] = [];

export default [
  {
    url: '/basic-api/table/CharacterList',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      //console.log(req.url);
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      const needRefresh = url.searchParams.get('NeedRefresh') || 'False';

      console.log('isOnline: ', url.searchParams.get('IsOnline'));
      if (needRefresh == 'True') {
        await RefreshCharacterList(
          Number(url.searchParams.get('IsOnline') || 2),
          url.searchParams.get('AccountID') || '',
          url.searchParams.get('Name') || '',
          url.searchParams.get('LoginIP') || '',
          url.searchParams.get('LoginMac') || '',
        );
      }

      res.setHeader('Content-Type', 'text/plain');
      res.statusCode = 200;
      res.end(
        JSON.stringify(
          resultPageSuccess(
            Number(url.searchParams.get('page')),
            Number(url.searchParams.get('pageSize')),
            CharacterList,
          ),
        ),
      );
    },
    // response: ({ url, query, headers, body }) => {
    //   const { page = 1, pageSize = 200 } = query;
    //   // console.log('url: ', url);
    //   // console.log('query: ', query);
    //   // console.log('headers: ', headers);
    //   // console.log('body: ', body);
    //   await RefreshCharacterList(0, query.AccountID, query.Name);
    //   //console.log('CharacterList: ', CharacterList);
    //   return resultPageSuccess(page, pageSize, CharacterList); //{ Name: '123', Index: 123 }
    // },
  },
  {
    url: '/basic-api/table/KickPlayer',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const data = {
        PlayerID: Number(url.searchParams.get('PlayerID')),
      };
      console.log(JSON.stringify(data));
      let result;
      try {
        const timeKey: number = Number(url.searchParams.get('TimeKey'));
        const timestamp = Date.parse(new Date().toString());
        console.log('时间戳差额：' + (timestamp - timeKey));
        if (timestamp - timeKey > 3000) {
          result = new MsgResult();
          result.StatusCode = 0;
          result.Message = 'key过期';
          res.statusCode = 400;
        } else {
          result = await SendManage(data, MsgType.KickPlayer);
        }
      } catch (error) {
        console.log(error);
        result = new MsgResult();
        result.StatusCode = 0;
        result.Message = '连接发生异常：' + error;
        res.statusCode = 400;
      } finally {
        console.log(result);
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
  {
    url: '/basic-api/table/BanPlayer',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const data = {
        AccountID: url.searchParams.get('AccountID'),
      };
      console.log(JSON.stringify(data));
      let result;
      try {
        const timeKey: number = Number(url.searchParams.get('TimeKey'));
        const timestamp = Date.parse(new Date().toString());
        console.log('时间戳差额：' + (timestamp - timeKey));
        if (timestamp - timeKey > 3000) {
          result = new MsgResult();
          result.StatusCode = 0;
          result.Message = 'key过期';
          res.statusCode = 400;
        } else {
          result = await SendManage(data, MsgType.BanPlayer);
        }
      } catch (error) {
        console.log(error);
        result = new MsgResult();
        result.StatusCode = 0;
        result.Message = '连接发生异常：' + error;
        res.statusCode = 400;
      } finally {
        console.log(result);
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
  {
    url: '/basic-api/table/UnbanPlayer',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const data = {
        AccountID: url.searchParams.get('AccountID'),
      };
      console.log(JSON.stringify(data));
      let result;
      try {
        const timeKey: number = Number(url.searchParams.get('TimeKey'));
        const timestamp = Date.parse(new Date().toString());
        console.log('时间戳差额：' + (timestamp - timeKey));
        if (timestamp - timeKey > 3000) {
          result = new MsgResult();
          result.StatusCode = 0;
          result.Message = 'key过期';
          res.statusCode = 400;
        } else {
          result = await SendManage(data, MsgType.UnbanPlayer);
        }
      } catch (error) {
        console.log(error);
        result = new MsgResult();
        result.StatusCode = 0;
        result.Message = '连接发生异常：' + error;
        res.statusCode = 400;
      } finally {
        console.log(result);
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
  {
    url: '/basic-api/table/SendHookQuestion',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const data = {
        AccountID: url.searchParams.get('AccountID'),
      };
      console.log(JSON.stringify(data));
      let result;
      try {
        const timeKey: number = Number(url.searchParams.get('TimeKey'));
        const timestamp = Date.parse(new Date().toString());
        console.log('时间戳差额：' + (timestamp - timeKey));
        if (timestamp - timeKey > 3000) {
          result = new MsgResult();
          result.StatusCode = 0;
          result.Message = 'key过期';
          res.statusCode = 400;
        } else {
          result = await SendManage(data, MsgType.SendHookQuestion);
        }
      } catch (error) {
        console.log(error);
        result = new MsgResult();
        result.StatusCode = 0;
        result.Message = '连接发生异常：' + error;
        res.statusCode = 400;
      } finally {
        console.log(result);
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
  {
    url: '/basic-api/table/ChangeHookSuspicionLevel',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const data = {
        AccountID: url.searchParams.get('AccountID'),
        Value: Number(url.searchParams.get('Value')),
      };
      console.log(JSON.stringify(data));
      let result;
      try {
        const timeKey: number = Number(url.searchParams.get('TimeKey'));
        const timestamp = Date.parse(new Date().toString());
        console.log('时间戳差额：' + (timestamp - timeKey));
        if (timestamp - timeKey > 3000) {
          result = new MsgResult();
          result.StatusCode = 0;
          result.Message = 'key过期';
          res.statusCode = 400;
        } else {
          result = await SendManage(data, MsgType.ChangeHookSuspicionLevel);
        }
      } catch (error) {
        console.log(error);
        result = new MsgResult();
        result.StatusCode = 0;
        result.Message = '连接发生异常：' + error;
        res.statusCode = 400;
      } finally {
        console.log(result);
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
  {
    url: '/basic-api/table/AddBannedAddress',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const data = {
        Address: url.searchParams.get('Address'),
        Type: Number(url.searchParams.get('Type')),
      };
      console.log(JSON.stringify(data));
      let result;
      try {
        const timeKey: number = Number(url.searchParams.get('TimeKey'));
        const timestamp = Date.parse(new Date().toString());
        console.log('时间戳差额：' + (timestamp - timeKey));
        if (timestamp - timeKey > 3000) {
          result = new MsgResult();
          result.StatusCode = 0;
          result.Message = 'key过期';
          res.statusCode = 400;
        } else {
          result = await SendManage(data, MsgType.AddBannedAddress);
        }
      } catch (error) {
        console.log(error);
        result = new MsgResult();
        result.StatusCode = 0;
        result.Message = '连接发生异常：' + error;
        res.statusCode = 400;
      } finally {
        console.log(result);
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
  {
    url: '/basic-api/table/ChangeServer',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const ServerName = url.searchParams.get('ServerName');
      console.log(ServerName);
      const result = new MsgResult();
      try {
        ChangeServer(ServerName as string);
        result.StatusCode = 1;
        result.Message = ServerName as string;
      } catch (error) {
        console.log(error);
        result.StatusCode = 0;
        result.Message = '修改失败' + error;
      } finally {
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
  {
    url: '/basic-api/table/GetServer',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      res.setHeader('Content-Type', 'text/plain');
      const result = new MsgResult();
      try {
        result.StatusCode = 1;
        result.Message = serverName();
      } catch (error) {
        console.log(error);
        result.StatusCode = 0;
        result.Message = '获取失败' + error;
      } finally {
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
] as MockMethod[];
