import { MockMethod } from 'vite-plugin-mock';
import { resultPageSuccess, resultSuccess } from '../_util';
import { databaseName, query } from './query';
import { MsgType, MsgResult, SendManage } from './socket';
// 接收一个sql语句 以及所需的values
// 这里接收第二参数values的原因是可以使用mysql的占位符 '?'
// 比如 query(`select * from my_database where id = ?`, [1])

async function RefreshDDropList(OwnerID = '', OwnerName = '') {
  let queryStr =
    "SELECT c.IndexID as PlayerID ,c.Name as PlayerName, replace(TagName,'DDROP@','') as MonsterName,Value FROM " +
    databaseName() +
    '.playerflags pt left join ' +
    databaseName() +
    ".characterinfo c on pt.playerId=c.IndexID where TagName like 'DDROP@%' ";
  if (OwnerID != '') {
    queryStr += " and c.IndexID='" + OwnerID + "'";
  }
  if (OwnerName != '') {
    queryStr += " and c.Name like '%" + OwnerName + "%'";
  }
  queryStr += ' limit 10000';
  console.log('QueryStr: ', queryStr);
  const results = await query(queryStr, '');
  //console.log('QueryCount: ', results.length);
  let i: number;
  for (i = 0; i < results.length; i++) {
    const data = results[i];
    //console.log('Data: ', data);
    if (DDropList.length > i) {
      DDropList[i] = data;
    } else {
      DDropList.push(data);
    }
  }
  if (DDropList.length > results.length) {
    DDropList.splice(results.length, DDropList.length - results.length);
  }
  //console.log('ResultCount: ', CharacterList.length);
  return results;
}

const DDropList: any[] = [];

export default [
  {
    url: '/basic-api/table/DDropList',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      //console.log(req.url);
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      const needRefresh = url.searchParams.get('NeedRefresh') || 'False';
      if (needRefresh == 'True') {
        await RefreshDDropList(
          url.searchParams.get('OwnerID') || '',
          url.searchParams.get('OwnerName') || '',
        );
      }
      res.setHeader('Content-Type', 'text/plain');
      res.statusCode = 200;
      res.end(
        JSON.stringify(
          resultPageSuccess(
            Number(url.searchParams.get('page')),
            Number(url.searchParams.get('pageSize')),
            DDropList,
          ),
        ),
      );
    },
  },
  {
    url: '/basic-api/table/EditDDrop',
    timeout: 100,
    method: 'get',
    rawResponse: async (req, res) => {
      const url = new URL(req.url || '', `http://${req.headers.host}`);
      res.setHeader('Content-Type', 'text/plain');
      const data = {
        PlayerID: Number(url.searchParams.get('PlayerID')),
        Value: Number(url.searchParams.get('Value')),
        MonsterName: url.searchParams.get('MonsterName'),
      };
      console.log(JSON.stringify(data));
      let result;
      try {
        const timeKey: number = Number(url.searchParams.get('TimeKey'));
        const timestamp = Date.parse(new Date().toString());
        console.log('时间戳差额：' + (timestamp - timeKey));
        if (timestamp - timeKey > 3000) {
          result = new MsgResult();
          result.StatusCode = 0;
          result.Message = 'key过期';
          res.statusCode = 400;
        } else {
          result = await SendManage(data, MsgType.EditDDrop);
        }
      } catch (error) {
        console.log(error);
        result = new MsgResult();
        result.StatusCode = 0;
        result.Message = '连接发生异常：' + error;
        res.statusCode = 400;
      } finally {
        console.log(result);
        res.end(JSON.stringify(resultSuccess(result)));
      }
    },
  },
] as MockMethod[];
