import { serverAddress } from './query';

export async function SendManage(
  data: any,
  msgType: MsgType,
  serverAds = serverAddress(),
): Promise<MsgResult> {
  const net = require('net');

  const msg = {
    Type: msgType,
    Data: data,
  };

  const client = net.connect(9991, serverAds);
  const result: MsgResult = new MsgResult();
  client.on('connect', () => {
    //console.log('连接到服务器');
    //向服务端发送数据
    console.log('发送管理消息：' + JSON.stringify(msg));
    client.write(JSON.stringify(msg));
  });

  //监听事件，当服务端发送数据过来时会触发该事件
  client.on('data', (data) => {
    try {
      if (result.StatusCode < 0) {
        console.log('data:' + data.toString());
        const j = JSON.parse(data);
        result.Message = j.Message;
        result.StatusCode = j.StatusCode;
      }
      client.end();
    } catch (e) {
      result.StatusCode = -1;
      result.Message = '返回值处理失败: ' + data;
    }
  });

  client.on('error', function (data: any) {
    result.StatusCode = 0;
    result.Message = '通讯过程中发生异常: ' + data;
    client.end();
  });

  client.on('end', () => {
    console.log('连接断开');
  });

  await Sleep(1000);

  return result;
}

export const Sleep = (ms) => {
  return new Promise((resolve) => setTimeout(resolve, ms));
};

export enum MsgType {
  GiveItem = 'GiveItem',
  EditItem = 'EditItem',
  DeleteItem = 'DeleteItem',
  SendItemToMail = 'SendItemToMail',
  SendItemToInventory = 'SendItemToInventory',
  EditDDrop = 'EditDDrop',
  KickPlayer = 'KickPlayer',
  BanPlayer = 'BanPlayer',
  UnbanPlayer = 'UnbanPlayer',
  SendHookQuestion = 'SendHookQuestion',
  ChangeHookSuspicionLevel = 'ChangeHookSuspicionLevel',
  AddBannedAddress = 'AddBannedAddress',
}

export class MsgResult {
  public StatusCode: number;
  public Message: string;
  constructor() {
    this.StatusCode = -1;
    this.Message = '未收到服务端回复！';
  }
}
